#! /usr/bin/perl
use strict;
use warnings;
use POSIX;
use CGI qw(fatalsToBrowser);

open LOG, "<", "/home/stream/count_datacube.log"
	or die "count_datacube.log: $!";

our %desc = (
	3013 => 'main-sd',
	3014 => 'main-hd',
	3015 => 'creativia-hd',
	3016 => 'creativia-sd',
	5013 => 'main-flash',
	5015 => 'creativia-flash',
);

my $lines = {};
my %streams = ();

while (<LOG>) {
	chomp;
	my ($date, $port, $proto, $audience, $count) = split /\s+/;
	next if (filter($port, $proto, $audience));
	my $stream_id = get_stream_id($port, $proto, $audience);
	$streams{$stream_id} = 1;
	$lines->{$date}{$stream_id} += $count;
}

close LOG;

print CGI::header(-type=>'image/png');

my $tmpfile = POSIX::tmpnam();
open GRAPH, ">", $tmpfile
	or die "$tmpfile: $!";
for my $date (sort keys %$lines) {
	my @cols = ();
	for my $stream (keys %{$lines->{$date}}) {
		push @cols, $lines->{$date}{$stream};
	}
	print GRAPH "$date ", join(' ', @cols), "\n";
}
close GRAPH;

my $tmpfile2 = POSIX::tmpnam();
open GNUPLOT, ">", $tmpfile2
	or die "$tmpfile2: $!";
print GNUPLOT "set terminal png\n";
print GNUPLOT "set xdata time\n";
print GNUPLOT "set timefmt \"20%y-%m-%d-%H:%M:%S\"\n";
print GNUPLOT "set xtics axis \"2000-00-00-01:00:00\"\n";
print GNUPLOT "set format x \"%H\"\n";
print GNUPLOT "set output '/dev/stdout'\n";

my @plots = ();
my $idx = 2;
for my $stream (keys %streams) {
	push @plots, "\"$tmpfile\" using 1:$idx title \"$stream\" with lines";
	++$idx;
}
print GNUPLOT "plot ", join(', ', @plots);

\"$tmpfile\" using 0:2 with lines, \"$tmpfile\" using 0:3 with lines\n";
close GNUPLOT;

system("gnuplot < $tmpfile2");

sub filter {
	my ($port, $proto, $audience) = @_;
	return 1 if (filter_list(CGI::param('port'), $port));
	return 1 if (filter_list(CGI::param('proto'), $proto));
	return 1 if (filter_list(CGI::param('audience'), $audience));
	return 0;
}

sub filter_list {
	my ($spec, $candidate) = @_;
	return 0 if ($spec eq 'compare' || $spec eq 'dontcare');
	my @spec = split /,/, $spec;
	for my $s (@spec) {
		return 0 if ($s eq $candidate);
	}
	return 1;
}	

sub get_stream_id {
	my ($port, $proto, $audience) = @_;
	my @keys = ();
	if (CGI::param('port') eq 'compare') {
		if (exists($desc{$port})) {
			push @keys, $desc{$port};
		} else {
			push @keys, "___" . $port . "___";
		}
	}
	push @keys, $proto if (CGI::param('proto') eq 'compare');
	push @keys, $audience if (CGI::param('audience') eq 'compare');
	return join(',', @keys);
}
